/* $Id: dialog.c,v 1.6 2007/07/15 22:05:22 mgdepalma Exp $
 *
 * Copyright (C) 2007 Softcraft Impresa, LLC <info@softcraft.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; You may only use version 2 of the License,
 * you have no option to use any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "dialog.h"
#include "language.h"

static struct _dialogue dialogue;    /* store alert dialog parts */
static struct _print_dialog printer; /* store print image dialog */
static struct _save_dialog story;    /* store save file dialog   */

static FileBinding binding[] = {
   { PNG,        "png" }
  ,{ JPEG,       "jpeg" }
  ,{ BMP,        "bmp" }
  ,{ PostScript, "ps" }
};

/*
 * return the file extension based of the given FileType index
 */
const gchar *
get_file_extension(FileType index)
{
  return binding[index].extension;
} /* </get_file_extension> */

/*
 * Add button to GtkDialog widget and optionally set image from an XPM file.
 */
GtkWidget *
add_button(GtkWidget *dialog, IconIndex index,
	      gchar *label, GtkResponseType handle)
{
  GtkWidget *button;
  gchar *padding = strargs("%6s", label);
  button = gtk_dialog_add_button (GTK_DIALOG(dialog), padding, handle);
  gtk_button_set_image (GTK_BUTTON(button), xpm_image(index));

  return button;
} /* </add_button> */

/*
 * remove button on a GtkDialog widget
 */
void
remove_button(GtkWidget *dialog, GtkWidget* button)
{
  GtkWidget *container = GTK_DIALOG(dialog)->action_area;
  gtk_container_remove(GTK_CONTAINER (container), button);
} /* </remove_button> */

/*
 * Assemble options menu for file types
 */
GtkWidget *
add_filetypes(GtkWidget **list)
{
  GtkWidget *selections, *label, *options, *menu, *item;

  selections = gtk_hbox_new (FALSE, 10);
  gtk_container_set_border_width (GTK_CONTAINER (selections), 10);
  gtk_widget_show (selections);

  label = xpm_label(ICON_PAINT, _L(LABEL_FILE_TYPE));
  gtk_box_pack_start (GTK_BOX (selections), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  options = *list = gtk_option_menu_new ();
  menu = gtk_menu_new ();
  
  item = make_menu_item (_L(LABEL_FILE_TYPE_PNG),
                         G_CALLBACK (set_file_type), GINT_TO_POINTER (PNG));
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
  
  item = make_menu_item (_L(LABEL_FILE_TYPE_JPG),
                         G_CALLBACK (set_file_type), GINT_TO_POINTER (JPEG));
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
  
  item = make_menu_item (_L(LABEL_FILE_TYPE_BMP),
                         G_CALLBACK (set_file_type), GINT_TO_POINTER (BMP));
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (options), menu);
  gtk_box_pack_start (GTK_BOX (selections), options, TRUE, TRUE, 0);
  gtk_widget_show (options);

  return selections;
} /* </add_filetypes> */

/*
 * callback to set file type
 */
void
set_file_type(GtkWidget *dialog, gpointer value)
{
  static gboolean program = TRUE;
  FileChooser *chooser = story.chooser;

  story.imgtype = GPOINTER_TO_INT(value);  /* get the file type from 'value' */

  if (program) {                        /* set the default save file name */
    gchar *name = (gchar*)gtk_entry_get_text (GTK_ENTRY(chooser->name));
    gchar savename[MAXPATH];
    gchar *scan;

    strcpy(savename, name);
    scan = g_strrstr(savename, ".");

    if (scan) {
      scan[0] = (gchar)0;  /* chop basename at the start of file extension */

      if (g_str_equal(savename, Program)) {
        name = strargs("%s.%s", Program, get_file_extension(story.imgtype));
        gtk_entry_set_text (GTK_ENTRY(chooser->name), name);
      }
      else {
        program = FALSE;
      }
    }
  }
  gtk_option_menu_set_history(GTK_OPTION_MENU (story.options), story.imgtype);
} /* </set_file_type> */

/*
 * Assemble options menu for paper size
 */
GtkWidget *
add_papersize(PaperSize *list)
{
  GtkWidget *options, *menu, *item;
  GtkWidget *layout = gtk_hbox_new(FALSE, 0);
  int idx;

  item = xpm_image(ICON_PAPER);
  gtk_box_pack_start (GTK_BOX (layout), item, TRUE, TRUE, 0);
  gtk_widget_show (item);

  options = gtk_option_menu_new ();
  menu = gtk_menu_new ();
  idx = 0;

  while (list[idx].name != NULL) {
    item = make_menu_item (list[idx].name,
                           G_CALLBACK (set_papersize), GINT_TO_POINTER (idx));
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    ++idx;
  }

  gtk_option_menu_set_menu (GTK_OPTION_MENU (options), menu);
  gtk_box_pack_start (GTK_BOX (layout), options, TRUE, TRUE, 0);
  gtk_widget_show (options);
  
  return layout;
} /* </add_papersize> */

/*
 * callback to set paper size
 */
void
set_papersize(GtkWidget *widget, gpointer value)
{
  printer.paper = GPOINTER_TO_INT(value);
  if (printer.paper != 0)
    unimplemented(printer.parent, NULL);
} /* </set_papersize> */

/*
 * Assemble options menu for available printers
 */
GtkWidget *
add_printers(GList *list)
{
  GtkWidget *layout = gtk_hbox_new(FALSE, 0);
  GtkWidget *options, *menu, *item;
  GList *iter;
  int idx;

  item = xpm_image(ICON_PRINTER);
  gtk_box_pack_start (GTK_BOX (layout), item, FALSE, FALSE, 5);
  gtk_widget_show (item);

  options = gtk_option_menu_new ();
  menu = gtk_menu_new ();
  idx = 0;

  for (iter = list; iter != NULL; iter = iter->next) {
    item = make_menu_item (iter->data,
                           G_CALLBACK(set_papersize), GINT_TO_POINTER (idx));
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
    idx++;
  }

  gtk_option_menu_set_menu (GTK_OPTION_MENU (options), menu);
  gtk_box_pack_start (GTK_BOX (layout), options, TRUE, TRUE, 0);
  gtk_widget_show (options);
  
  return layout;
} /* </add_printers> */

/*
 * callback to set the printer
 */
void
set_printer(GtkWidget *widget, gpointer value)
{
  GList *iter;
  GList *printers = get_printers_list();
  int index = GPOINTER_TO_INT(value);
  int mark = 0;

  for (iter = printers; iter != NULL; iter = iter->next) {
    if (mark == index) break;
    ++mark;
  }
} /* </set_printer> */

/*
 * add_horizontal_separator convenience function
 */
void
add_horizontal_separator(GtkWidget* layout)
{
  GtkWidget *separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (layout), separator, FALSE, TRUE, 0);
  gtk_widget_show (separator);
}

/*
 * add_horizontal_space convenience function
 */
void
add_horizontal_space(GtkWidget* layout, gint space)
{
  GtkWidget *area  = gtk_hbox_new(TRUE, space);
  GtkWidget *label = gtk_label_new (" ");

  gtk_box_pack_start (GTK_BOX (layout), area, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (area), label, TRUE, TRUE, space);

  gtk_widget_show (label);
  gtk_widget_show (area);
} /* </add_horizontal_space> */

/*
 * add_vertical_space convenience function
 */
void
add_vertical_space(GtkWidget* layout, gint space)
{
  GtkWidget *area  = gtk_vbox_new(TRUE, space);
  GtkWidget *label = gtk_label_new (" ");

  gtk_box_pack_start (GTK_BOX (layout), area, FALSE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (area), label, TRUE, TRUE, space);

  gtk_widget_show (label);
  gtk_widget_show (area);
} /* </add_vertical_space> */

/*
 * make_menu_item convenience function
 */
GtkWidget *
make_menu_item (gchar *name, GCallback callback, gpointer data)
{
  GtkWidget *item;
  
  item = gtk_menu_item_new_with_label (name);
  g_signal_connect (G_OBJECT(item), "activate", callback, data);
  gtk_widget_show (item);

  return item;
} /* </make_menu_item> */

/*
 * handler for selecting print mode in {PRINT_DEVICE, PRINT_FILE}
 */
static void
print_device(GtkWidget *button, PrintDialog *print)
{
  GtkWidget *dialog = print->dialog;

  gint width  = print->width;	/* mininal print->dialog size */
  gint height = print->height;


  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(button))) {
    print->device = PRINT_FILE;

    gtk_widget_show (print->filer);
    gtk_entry_set_editable (GTK_ENTRY(print->command), FALSE);

    gtk_widget_set_usize (dialog, width, 3*height);
    gdk_window_resize (dialog->window, width, 3*height);

    gtk_window_set_resizable (GTK_WINDOW(dialog), TRUE);
  }
  else {
    print->device = PRINT_DEVICE;

    gtk_widget_hide (print->filer);
    gtk_entry_set_editable (GTK_ENTRY(print->command), TRUE);

    gtk_widget_set_usize (dialog, width, height);
    gdk_window_resize (dialog->window, width, height);

    gtk_window_set_resizable (GTK_WINDOW(dialog), FALSE);
  }
} /* </print_device> */

/*
 * (private) select print orientation in {PRINT_LANDSCAPE, PRINT_PORTRAIT}
 */
static void
print_orientation(GtkWidget *button, gpointer value)
{
  switch (GPOINTER_TO_INT (value)) {
    case PRINT_LANDSCAPE:
      printer.landscape = TRUE;
      unimplemented(printer.parent, _L(LABEL_PRINT_LANDSCAPE));
      break;

    case PRINT_PORTRAIT:
      printer.landscape = FALSE;
      break;
  }
} /* </print_orientation> */

/*
 * (private) create file chooser callback method
 */
static gboolean
printdialog_callback(GtkWidget *button, PrintDialog *print)
{
  gboolean status = TRUE;

  if (print->callback)	/* callback can veto status */
    status = (*print->callback) (print);
  else
    unimplemented(print->dialog, NULL);

  if (status) {
    gtk_widget_hide (print->dialog);
  }

  return status;
} /* </printdialog_callback> */

/*
 * (private) create print dialog
 */
static PrintDialog *
printdialog_create(GtkWidget *parent, GtkFunction callback)
{
  const gint width  = 400;
  const gint height = 120;

  GtkWidget *dialog = gtk_dialog_new ();
  GtkWidget *layout = gtk_vbox_new (FALSE, 2);  /* print dialog layout */

  GtkWidget *selector = add_printers (get_printers_list ());
  GtkWidget *area, *button, *field, *filer, *frame;

  FileChooser *chooser = filechooser_new (FILECHOOSER_CWD);
  gchar *savefile;


  /* Customize behavior and look */
  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
  gtk_widget_set_usize (dialog, width, height);

  gtk_window_set_title (GTK_WINDOW (dialog), _L(LABEL_PRINT));
  gtk_window_set_icon (GTK_WINDOW (dialog), xpm_pixbuf(ICON_PRINT, NULL));
  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
  gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

  /* Construct print command area */
  frame = gtk_frame_new(NULL);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), frame);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
  gtk_container_set_border_width(GTK_CONTAINER (frame), 2);
  gtk_widget_show(frame);

  gtk_container_add(GTK_CONTAINER(frame), layout);
  gtk_widget_show(layout);

  /* Add container for available printers */
  area = gtk_hbox_new (FALSE, 5);
  gtk_box_pack_start (GTK_BOX(layout), area, FALSE, FALSE, 0);
  gtk_widget_show (area);

  /* Add available printers */
  gtk_box_pack_start (GTK_BOX(area), selector, FALSE, TRUE, 5);
  gtk_widget_set_usize (selector, 7*width/9, 0);
  gtk_widget_show(selector);

  /* Print command is invisible to the user (for now) */
  field = printer.command = gtk_entry_new();
  gtk_entry_set_text (GTK_ENTRY(field), "lpr");

  /* Add save to file  */
  area = gtk_hbox_new (FALSE, 2);
  gtk_box_pack_start (GTK_BOX(layout), area, FALSE, FALSE, 5);
  gtk_widget_show(area);

  /* Create check box for printing to a file */
  add_horizontal_space (area, width/3);
  button = gtk_check_button_new_with_label(_L(LABEL_PRINT_TO_FILE));
  gtk_box_pack_start (GTK_BOX(area), button, FALSE, TRUE, 0);
  g_signal_connect (G_OBJECT(button), "toggled",
                          G_CALLBACK (print_device), &printer);
  printer.outfile = button;
  gtk_widget_show(button);

  /* Setup save to file chooser */
  filer = filechooser_layout (chooser);
  gtk_box_pack_start (GTK_BOX(layout), filer, TRUE, TRUE, 5);

  /* Adjust size of chooser->name entry field. */
  gtk_widget_set_usize (chooser->name, 7*width/9, 0);

  /* Initialize default output file name */
  savefile = strargs("%s.%s", Program, get_file_extension(PostScript));
  gtk_entry_set_text(GTK_ENTRY(chooser->name), savefile);

  /* Assemble the buttons */
  button = add_button(dialog, ICON_CANCEL, _L(LABEL_CANCEL),
                               GTK_RESPONSE_CANCEL);

  g_signal_connect_swapped (G_OBJECT (button), "clicked",
                            GTK_SIGNAL_FUNC (gtk_widget_hide),
                            G_OBJECT (dialog));
  gtk_widget_show(button);

  button = add_button(dialog, ICON_PRINT, _L(LABEL_PRINT), GTK_RESPONSE_OK);
  g_signal_connect (G_OBJECT(button), "clicked",
                         G_CALLBACK (printdialog_callback), &printer);
  gtk_widget_show(button);

  /* Set event handlers for window close/destroy */
  g_signal_connect (G_OBJECT (dialog), "destroy",
                      G_CALLBACK (gtk_widget_hide), NULL);

  g_signal_connect (G_OBJECT (dialog), "delete_event",
                      G_CALLBACK (gtk_widget_hide), NULL);

  /* One time initialization of struct _print_dialog */
  printer.callback= callback;
  printer.chooser = chooser;

  printer.parent  = parent;
  printer.dialog  = dialog;
  printer.filer   = filer;

  printer.width   = width;
  printer.height  = height;

  printer.device  = PRINT_DEVICE;
  printer.paper   = 0;		/* PaperSize Paper[] index (see, print.c) */

  return &printer;
} /* </printdialog_create> */

/*
 * print image dialog wrapper
 */
PrintDialog *
printdialog_new (GtkWidget *parent, GtkFunction agent)
{
  static PrintDialog *self = NULL;
  return (self != NULL) ? self : (self = printdialog_create(parent, agent));
} /* </printdialog> */

/*
 * (private) create file chooser callback method
 */
static gboolean
savedialog_callback(GtkWidget *button, SaveDialog *save)
{
  //SaveDialog *save = (SaveDialog *)data;
  const gchar *outfile = filechooser_get_selected_name (save->chooser);
  gboolean status = TRUE;
  struct stat info;

  if (lstat(outfile, &info) == 0 && S_ISDIR(info.st_mode))
    status = FALSE;
  else if ( (status = (*save->callback) (save)) )
    gtk_widget_hide (save->dialog);

  return status;
} /* </savedialog_callback> */

/*
 * (private) create file chooser dialog
 */
static SaveDialog *
savedialog_create(GtkWidget *parent, GtkFunction callback)
{
  const gint width  = 400;
  const gint height = 300;

  GtkWidget *dialog   = gtk_dialog_new ();
  GtkWidget *layout   = gtk_vbox_new (FALSE, 2);  /* save dialog layout */

  GtkWidget *selector = add_filetypes(&(story.options));
  GtkWidget *apply, *cancel, *filer, *frame;

  FileChooser *chooser = filechooser_new (FILECHOOSER_CWD);
  gchar *savefile;

  /* Customize behavior and look */
  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
  gtk_widget_set_usize (dialog, width, height);

  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
  gtk_window_set_icon (GTK_WINDOW (dialog), xpm_pixbuf(ICON_SAVE, NULL));
  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
  /* gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE); */
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

  /* Set event handlers for window close/destroy */
  g_signal_connect (G_OBJECT (dialog), "destroy",
	              G_CALLBACK (gtk_widget_hide), NULL);

  g_signal_connect (G_OBJECT (dialog), "delete_event",
	 	      G_CALLBACK (gtk_widget_hide), NULL);

  /* Assemble contents of dialog window */
  frame = gtk_frame_new(NULL);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), frame);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_ETCHED_IN);
  gtk_container_set_border_width(GTK_CONTAINER (frame), 2);
  gtk_widget_show(frame);

  gtk_container_add(GTK_CONTAINER(frame), layout);
  gtk_widget_show(layout);

  filer = filechooser_layout (chooser);
  gtk_box_pack_start (GTK_BOX(layout), filer, TRUE, TRUE, 5);
  gtk_widget_show (filer);

  /* Adjust size of chooser->name entry field. */
  gtk_widget_set_usize (chooser->name, 8*width/9, 0);

  /* Initialize default output file name */
  savefile = strargs("%s.%s", Program, get_file_extension(PNG));
  gtk_entry_set_text(GTK_ENTRY(chooser->name), savefile);

  /* Add file types selection */
  gtk_box_pack_start (GTK_BOX(layout), selector, FALSE, FALSE, 0);
  gtk_widget_show(selector);

  /* Add CANCEL and APPLY buttons */
  cancel = add_button(dialog, ICON_CANCEL, _L(LABEL_CANCEL),
                        GTK_RESPONSE_CANCEL);

  /* Connect the cancel button to hide the widget */
  g_signal_connect_swapped (G_OBJECT (cancel), "clicked",
                            G_CALLBACK (gtk_widget_hide),
                            G_OBJECT (dialog));

  apply = add_button(dialog, ICON_SAVE, _L(LABEL_SAVE), GTK_RESPONSE_OK);
  gtk_signal_connect (GTK_OBJECT (apply), "clicked",
                       G_CALLBACK (savedialog_callback), &story);

  /* One time initialization of module data structures */
  story.parent  = parent;
  story.dialog  = dialog;
  story.filer   = filer;

  story.callback= callback;
  story.chooser = chooser;
  story.imgtype = PNG;

  story.cancel  = cancel;
  story.apply   = apply;

  return &story;
} /* </savedialog_create> */

/*
 * save file chooser dialog wrapper
 */
SaveDialog *
savedialog_new (GtkWidget *parent, GtkFunction callback)
{
  static SaveDialog *self = NULL;
  return (self != NULL) ? self : (self = savedialog_create(parent, callback));
} /* </savedialog> */

/*
 * alert dialog with CANCEL and OK buttons
 */
static GtkWidget *
alert(GtkWidget *parent, IconIndex icon, const gchar *message)
{
  GtkWidget *button;
  GtkWidget *dialog = gtk_dialog_new ();
  GtkWidget *layout = gtk_hbox_new (FALSE, 8);
  GtkWidget *label  = xpm_label (icon, message);

  /* Customize behavior and look */
  gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);

  gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
  gtk_window_set_resizable(GTK_WINDOW(dialog), FALSE);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  /*
  gtk_window_set_role(GTK_WINDOW(dialog), "notify_dialog");
  g_signal_connect(G_OBJECT(dialog), "response",
                      G_CALLBACK(fire_alert), dialog);
  */

  /* Assemble contents of dialog window with icon and message */
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), layout);
  gtk_box_pack_start(GTK_BOX(layout), label, FALSE, TRUE, 5);
  gtk_widget_show (layout);

  /* Add dismiss (acknowledge) button */
  button = add_button(dialog, ICON_CANCEL,_L(LABEL_CANCEL),GTK_RESPONSE_CANCEL);
  gtk_widget_hide (button);  /* must be enabled elsewhere */

  /* Store major components in class dialogue */
  dialogue.window = dialog;
  dialogue.layout = layout;
  dialogue.cancel = button;
  dialogue.apply  =
     add_button(dialog, ICON_DONE, _L(LABEL_OK), GTK_RESPONSE_OK);

  return dialog;
} /* </alert> */

/*
 * alert wrapper method
 */
static GtkWidget *
message(GtkWidget *parent, IconIndex icon, const gchar *text)
{
  static GtkWidget *dialog = NULL;

  if (dialog == NULL)    /* GTK object oriented nature bites on reuse */
    dialog = alert(parent, icon, text);
  else {
    GtkWidget *container = GTK_DIALOG(dialog)->vbox;
    GtkWidget *layout    = gtk_hbox_new (FALSE, 8);
    GtkWidget *label     = xpm_label(icon, text);

    gtk_container_remove(GTK_CONTAINER (container), dialogue.layout);
    gtk_box_pack_start(GTK_BOX (layout), label , FALSE, FALSE, 2);
    gtk_container_add(GTK_CONTAINER (container), layout);
    gtk_widget_show (layout);

    dialogue.layout = layout;
  }
  return dialog;
} /* </message> */

/*
 * fire up an alert with a given icon and message
 */
gint
notice(GtkWidget *parent, IconIndex icon, const gchar *text)
{
  gint status;
  GtkWidget *dialog = message(parent, icon, text);

  gtk_widget_show (dialog);
  status = gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_hide (dialog);

  return status;
} /* </notice> */

/*
 * notify dialog for file write over
 */
gint
overwrite(GtkWidget *parent, const gchar* filename)
{
  GtkWidget *dialog, *apply;
  gchar *text = strargs("%s: %s", filename, _L(LABEL_FILE_EXIST));
  gint status;

  dialog = message(parent, ICON_QUESTION, text);
  apply  = dialogue.apply;

  gtk_widget_show (dialogue.cancel);
  gtk_button_set_image(GTK_BUTTON (apply), xpm_image(ICON_SAVE));
  gtk_button_set_label(GTK_BUTTON (apply), _L(LABEL_WRITE_OVER));

  gtk_widget_show (dialog);
  status = gtk_dialog_run (GTK_DIALOG (dialog));

  gtk_widget_hide (dialogue.cancel);
  gtk_button_set_image(GTK_BUTTON (apply), xpm_image(ICON_DONE));
  gtk_button_set_label(GTK_BUTTON (apply), _L(LABEL_OK));
  gtk_widget_hide (dialog);

  return status;
} /* </overwrite> */

/*
 * fire up an alert saying feature is not implemented
 */
void
unimplemented(GtkWidget *parent, const gchar* feature)
{
  static IconIndex icon = ICON_WARNING;

  if (feature != NULL) {
    gchar *message = strargs("[%s] %s", feature, _L(LABEL_UNIMPLEMENTED));
    notice(parent, icon, message);
  }
  else {
    notice(parent, icon, _L(LABEL_UNIMPLEMENTED));
  }
} /* </unimplemented> */
