/* $Id: gsnapshot.c,v 1.5 2007/07/15 22:05:23 mgdepalma Exp $
 *
 * Copyright (C) 2007 Softcraft Impresa, LLC <info@softcraft.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; You may only use version 2 of the License,
 * you have no option to use any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gsnapshot.h"
#include "language.h"

static gsnapshot_t snapshot;	/* (protected) encapsulated program data */

const char *Program;		/* (public) published program name */
const char *Release;		/* (public) published program version */

/*
 * (private) about
 */
static gboolean
about(GtkWidget *widget, gpointer data)
{
  static char *Information =
  "is a gtk-based utility which captures screen contents.\n"
  "One can select to capture the entire screen, a window or a rectangle.\n"
  "The snapshot can be saved to a file and/or printed.\n"
  "\n"
  "The program is developed by Mauro DePalma and distributed\n"
  "under the terms and condition of the GNU Public License.\n";

  notice(snapshot->window, ICON_SNAPSHOT,
                 strargs("\n%s %s %s", Program, Release, Information));
  return FALSE;
} /* </about> */

/*
 * clean application exit
 */
static gboolean
finis(GtkWidget *widget, gpointer data)
{
  gtk_main_quit ();
  return FALSE;
} /* </finis> */

/*
 * refresh view of the canvas area
 */
static void
refresh(GtkWidget *canvas, gpointer data)
{
  GdkPixbuf *image = snapshot->image;

  if (image) {
    redraw_pixbuf (canvas, image);
  }
} /* </refresh> */

/*
 * callback to print image
 */
static gboolean
printimage(gpointer data)
{
  FILE *out;	/* either a file or pipe to print->command */
  PrintDialog  *print = (PrintDialog *)data;
  gboolean status = TRUE;

  if (print->device == PRINT_FILE) {
    const gchar *outfile = filechooser_get_selected_name (print->chooser);

    if (access(outfile, F_OK) == 0) {  /* file already exists */
      if (overwrite(print->dialog, outfile) != GTK_RESPONSE_OK)
        status = FALSE;
    }

    if (status) {
      out = fopen(outfile, "w");
      print_pixbuf (out, snapshot->image, print->paper);
      fclose(out);

      notice(print->dialog, ICON_INFO, strargs("%s: PostScript", outfile));
    }
  }
  else {
    const gchar *command = gtk_entry_get_text (GTK_ENTRY(print->command));

    if (strlen(command) < 2 || strncmp(command, "lp", 2) != 0) {
      notice(print->dialog, ICON_ERROR, _L(LABEL_PRINT_INVALID));
    }
    else {
      out = popen(command, "w");
      print_pixbuf (out, snapshot->image, print->paper);
      pclose(out);

      notice(print->dialog, ICON_INFO,
             strargs("%s: %s", _L(LABEL_PRINT_JOB_SENT), command));
    }
  }
  return status;
} /* </printimage> */

/*
 * write out snapshot to a file (bmp, jpeg, png)
 */
static gboolean
savefile(gpointer data)
{
  SaveDialog *save = (SaveDialog *)data;
  gboolean status = TRUE;

  const gchar *format = get_file_extension(save->imgtype);
  const gchar *filename = filechooser_get_selected_name (save->chooser);

  if (access(filename, F_OK) != 0)  /* file does not exist */
    gdk_pixbuf_save(snapshot->image, filename, format, NULL, NULL);
  else {
    if (overwrite(save->dialog, filename) == GTK_RESPONSE_OK)
      gdk_pixbuf_save(snapshot->image, filename, format, NULL, NULL);
    else
      status = FALSE;
  }
  return status;
} /* </savefile> */

/*
 * screenshot callback
 */
static void
screenshot(GtkWidget *widget, gpointer data)
{
  gint xpos, ypos;
  GtkWidget *window = snapshot->window;

  /* Remember the main window position and hide */
  if (snapshot->hide) {
    gtk_window_get_position (GTK_WINDOW(window), &xpos, &ypos);
    gtk_widget_hide (window);
    gdk_display_sync (gtk_widget_get_display (window));
  }

  /* Capture in the specified mode and store image in snapshot->image */
  if (snapshot->delay > 0) grab_pointer_sleep (snapshot->delay);
  snapshot->image = capture(snapshot->mode, snapshot->decorations);
  if (data) refresh(snapshot->viewer, NULL);

  /* Restore the main window original position */
  if (snapshot->hide) {
    gtk_window_move (GTK_WINDOW(window), xpos, ypos);
    gtk_widget_show (window);
  }
} /* </screenshot> */

/*
 * save_as callback
 */
static void
save_as(GtkWidget *widget, gpointer data)
{
  SaveDialog *save = snapshot->save;
  gtk_widget_show (save->dialog);
} /* </save_as> */

/*
 * print to file callback
 */
static void printofile(GtkWidget *widget, gpointer data)
{
  PrintDialog *print = snapshot->print;
  gtk_widget_show (print->dialog);
} /* </printofile> */

/* 
 * (private) set_capture_mode
 * (private) set_snapshot_delay
 * (private) set_window_decorations
 * (private) set_application_hide
 */
static void set_capture_mode(GtkWidget *item, gpointer value)
{
  snapshot->mode = GPOINTER_TO_INT(value);
} /* </set_capture_mode> */

static void set_snapshot_delay(GtkAdjustment *adjust)
{
  snapshot->delay = (unsigned int)adjust->value;
} /* </set_snapshot_delay> */

static void
set_window_decorations(GtkToggleButton *button)
{
  snapshot->decorations = button->active;
} /* </set_window_decorations> */

static void
set_application_hide(GtkToggleButton *button)
{
  snapshot->hide = button->active;
} /* </set_application_hide> */

/*
 * construct_viewer - construct view and control area
 */
static void
construct_viewer(GtkWidget *layout, GtkWidget *window)
{
  GtkWidget *button;
  GtkWidget *canvas;
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *box;

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (layout), hbox, FALSE, TRUE, 0);
  gtk_widget_show (hbox);

  /* Assemble display canvas area */
  canvas = snapshot->viewer = gtk_drawing_area_new ();
  gtk_widget_set_size_request (GTK_WIDGET (canvas), VIEW_WIDTH, VIEW_HEIGHT);
  gtk_box_pack_start (GTK_BOX (hbox), canvas, FALSE, TRUE, 0);
  gtk_widget_show (canvas);

  /* Signals used to handle backing pixmap */
  g_signal_connect (G_OBJECT (canvas), "expose_event",
		    G_CALLBACK (refresh), NULL);

  /* Assemble vbox for CAPTURE, SAVE, PRINT buttons */
  vbox = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, TRUE, BOX_SPACING);
  gtk_widget_show (vbox);

  button = xpm_button(ICON_CAMERA, _L(LABEL_CAPTURE));
  gtk_box_pack_start (GTK_BOX (vbox), button, FALSE, TRUE, 0);
  g_signal_connect_swapped (G_OBJECT (button), "clicked",
                            G_CALLBACK (screenshot),
                            G_OBJECT (window));
  gtk_widget_show (button);

  box = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), box, FALSE, TRUE, 20);
  gtk_widget_show (box);

  button = xpm_button(ICON_SAVE_AS, strargs("%s...", _L(LABEL_SAVE_AS)));
  gtk_box_pack_start (GTK_BOX (box), button, FALSE, TRUE, 0);
  g_signal_connect_swapped (G_OBJECT (button), "clicked",
                            G_CALLBACK (save_as),
                            G_OBJECT (window));
  gtk_widget_show (button);

  button = xpm_button(ICON_PRINT, strargs("%s...", _L(LABEL_PRINT)));
  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
  g_signal_connect_swapped (G_OBJECT (button), "clicked",
                            G_CALLBACK (printofile),
                            G_OBJECT (window));
  gtk_widget_show (button);
} /* </construct_viewer> */

/*
 * construct_modes - construct modes area
 */
static void
construct_modes(GtkWidget *layout, GtkWidget *window)
{
  GtkWidget *nobs;    /* widgets for mode, delay and decorations */

  GtkWidget *box, *button, *label;
  GtkWidget *options, *menu, *item;

  GtkObject *adjust;
  GtkWidget *scale;


  /* Assemble container
  */
  nobs = gtk_vbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (layout), nobs, FALSE, TRUE, 0);
  gtk_widget_show (nobs);

  /* Assemble options menu for capture mode
  */
  box = gtk_hbox_new (FALSE, BORDER_WIDTH);
  gtk_container_set_border_width (GTK_CONTAINER (box), BORDER_WIDTH);
  gtk_box_pack_start (GTK_BOX (nobs), box, FALSE, TRUE, 0);
  gtk_widget_show (box);

  label = gtk_label_new (_L(LABEL_GRAB));
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  options = gtk_option_menu_new ();
  menu = gtk_menu_new ();
  
  item = make_menu_item (_L(LABEL_GRAB_SCREEN),
                         G_CALLBACK (set_capture_mode),
                         GINT_TO_POINTER(GRAB_SCREEN));
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
  
  item = make_menu_item (_L(LABEL_GRAB_WINDOW),
                         G_CALLBACK (set_capture_mode),
                         GINT_TO_POINTER (GRAB_WINDOW));
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
  
  item = make_menu_item (_L(LABEL_GRAB_REGION),
                         G_CALLBACK (set_capture_mode),
                         GINT_TO_POINTER (GRAB_REGION));
  gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

  gtk_option_menu_set_menu (GTK_OPTION_MENU (options), menu);
  gtk_box_pack_start (GTK_BOX (box), options, TRUE, TRUE, 0);
  gtk_widget_show (options);

  /* Assemble slider to control snapshot delay
  */
  box = gtk_hbox_new (FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (box), 0);
  gtk_box_pack_start (GTK_BOX (nobs), box, FALSE, TRUE, 0);
  gtk_widget_show (box);

  label = gtk_label_new (_L(LABEL_DELAY));
  gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  adjust = gtk_adjustment_new (0.0, 0.0, 20.0, 1.0, 1.0, 0.0);
  g_signal_connect (G_OBJECT (adjust), "value_changed",
                    G_CALLBACK (set_snapshot_delay), NULL);

  scale = gtk_hscale_new (GTK_ADJUSTMENT (adjust));
  gtk_scale_set_digits (GTK_SCALE (scale), 0);
  gtk_box_pack_start (GTK_BOX (box), scale, TRUE, TRUE, 0);
  gtk_widget_show (scale);

  /* Assemble checkbutton to control including window decorations */
  box = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (nobs), box, TRUE, TRUE, 0);
  gtk_widget_show (box);

  button = gtk_check_button_new_with_label(_L(LABEL_DECORE));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
                                snapshot->decorations);

  g_signal_connect (G_OBJECT (button), "toggled",
                    G_CALLBACK (set_window_decorations), NULL);

  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
  gtk_widget_show (button);

  /* Assemble checkbutton to control hiding application window */
  button = gtk_check_button_new_with_label(_L(LABEL_HIDE));
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button),
                                snapshot->hide);

  g_signal_connect (G_OBJECT (button), "toggled",
                    G_CALLBACK (set_application_hide), NULL);

  gtk_box_pack_start (GTK_BOX (box), button, TRUE, TRUE, 0);
  gtk_widget_show (button);
} /* </construct_modes> */

/*
 * (private) construct_trail - construct trail area
 */
static void
construct_trail(GtkWidget *layout, GtkWidget *window)
{
  GtkWidget *info;   /* programme information label */
  GtkWidget *quit;   /* programme clean exit button */
  GtkWidget *trail;  /* trailer hbox (info, quit)   */

  trail = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX (layout), trail, FALSE, TRUE, 4);
  gtk_widget_show (trail);

  info = xpm_button(ICON_HELP, _L(LABEL_INFO));
  gtk_box_pack_start (GTK_BOX (trail), info, FALSE, FALSE, 5);
  gtk_button_set_relief (GTK_BUTTON (info), GTK_RELIEF_NONE);
  g_signal_connect (G_OBJECT(info), "clicked", G_CALLBACK(about), NULL);
  gtk_widget_show (info);

  /* Assemble the quit button.. we are done! */
  quit = xpm_button(ICON_CLOSE, _L(LABEL_CLOSE));
  gtk_box_pack_end (GTK_BOX (trail), quit, FALSE, TRUE, 9);
  /* gtk_button_set_relief (GTK_BUTTON (quit), GTK_RELIEF_NONE); */
  g_signal_connect (G_OBJECT(quit), "clicked", G_CALLBACK(finis), NULL);

  GTK_WIDGET_SET_FLAGS (quit, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (quit);

  gtk_widget_show (quit);
} /* </construct_trail> */

/*
 * interface - construct user interface
 */
GtkWidget *
interface(GtkWidget *window)
{
  GtkWidget *layout;                   /* Interface main layout */

  layout = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), layout);
  gtk_widget_show (layout);

  construct_viewer(layout, window);    /* Assemble viewer area */
  add_vertical_space(layout, BOX_SPACING);
  add_horizontal_separator(layout);

  construct_modes(layout, window);     /* Assemble modes area */
  add_horizontal_separator(layout);

  construct_trail(layout, window);     /* Assemble trail area */
  return layout;
} /* </interface> */

/*
 * main - main program implementation
 */
int main(int argc, char *argv[])
{
  GtkWidget *window;        /* GtkWidget is the storage type for widgets */
  struct _gsnapshot memory;

  Program = basename(argv[0]);
  Release = "1.0";

  /* Initialization of the GTK */
  gtk_set_locale ();
  gtk_init (&argc, &argv);

  /* Create a new top level window */
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  /* Initialize program data structure */
  snapshot = &memory;

  snapshot->decorations = FALSE;
  snapshot->delay       = 0;
  snapshot->hide        = TRUE;
  snapshot->image       = NULL;
  snapshot->mode        = GRAB_SCREEN;
  snapshot->window      = window;
  snapshot->layout      = interface (window);
  snapshot->print       = printdialog_new (window, printimage);
  snapshot->save        = savedialog_new (window, savefile);

  /* Set title and border width of the main window */
  gtk_window_set_title (GTK_WINDOW (window), Program);
  gtk_window_set_icon (GTK_WINDOW (window), xpm_pixbuf(ICON_TITLE, NULL));
  gtk_container_set_border_width (GTK_CONTAINER (window), BORDER_WIDTH);
  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
  gtk_window_set_resizable (GTK_WINDOW(window), FALSE);

  /* Set event handlers for window close/destroy */
  g_signal_connect (G_OBJECT (window), "destroy",
	              G_CALLBACK (finis), NULL);

  g_signal_connect (G_OBJECT (window), "delete_event",
	 	      G_CALLBACK (finis), NULL);

  /* Realize the visual components */
  screenshot(window, NULL);  /* Initialize snapshot->image with screenshot */
  gtk_widget_show (window);

  /* Main event loop */
  gtk_main ();
    
  return Success;
} /* </gsnapshot> */
